package checkers;

/*
 * Created on Mar 2, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
 
import java.util.Hashtable;
import java.util.ArrayList;
/**
 * @author daltonj
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class BitBoard {
	
	public final static int DEBUG = 0;
	public final static long MAX = 0x7ffffffffL;
	public final static long MASK = 0x7FBFDFEFFL;
	public final static long MIN = 0;
	public ArrayList nStates;
	
	public boolean whiteMove;
	
	// position of pieces
	private long FA; // black on black's turn, white kings on white's turn
	private long BA; // white on white's turn, black kings on black's turn
	private long FP; // fowards passive (i.e. black on white's turn, white kings on black's turn 
	private long BP; // backwards passive (i.e. white on black's turn, black kings on white's move
	
	//board state
	protected long B; // black pieces
	protected long W; // white pieces
	protected long K; // all kings
	
	// Possible moves
	private long RF; // right forward moves
	private long LF; // left forward moves
	private long LB; // left backwards moves
	private long RB; // right backwards moves
	
	public int jumps; // sign bit says if there a jump available
	
	private long EMPTY; // holds the empty locations on the board
	
	private static Hashtable Hashy;
	
	public BitBoard(){
		B = 0x00001EFFL;
		W = 0x7FBC00000L;
		K = 0x0L;
		whiteMove = false;	
		EMPTY = ~(B | W);
		setup();
		jumps = doJumps();
	}
	
	
	public BitBoard(State s, Hashtable in){
			B = s.B & MASK;
			W = s.W & MASK;
			K = s.K & MASK;
			whiteMove = s.whiteMove;
			setup();
			jumps = doJumps();
			Hashy = in;
			if( jumps >0){
				nStates = createJumpStates();
				if(DEBUG==1)TestBitBoard.outputState(this);
				nStates = doMultiJump(nStates);
			}
			else{
				possibleSingleMoves();
				nStates = createSingleStates();
				
			}
				
	}
	
	public BitBoard(State s, Hashtable in, int i){
		B = s.B & MASK;
		W = s.W & MASK;
		K = s.K & MASK;
		whiteMove = s.whiteMove;
		setup();
		jumps = doJumps();
		Hashy = in;
		
		}
	
	private ArrayList doMultiJump(ArrayList states){
		for(int i=0; i<states.size();i++){
			State nState = (State)states.get(i); //get the current state
			boolean deleteVal = false; // don't delete it, yet
			BitBoard s = new BitBoard(nState, Hashy, 1); //create the new BitBoard
				if (!whiteMove){ 					//whites turn
					long moveMask = (s.B ^ B) & s.B;
					long kingMask = (s.B^B) & B;
					boolean isDouble = ((kingMask & K)>0 || (moveMask & 0x780000000L)==0);
					long me = s.DoPossibleJumpPieces();
					if((me & moveMask) > 0  && isDouble){
						ArrayList tmp = s.createJumpStates();
						for(int j=0; j<tmp.size();j++){
							State temp = (State)tmp.get(j);
							if ((temp.B & moveMask) == 0){
								states.add(temp);
								deleteVal = true;
							}
						}
					}//end found possibles if
					if(deleteVal) {states.remove(i); i--;}
				} //end white 
				else{
					long moveMask = (s.W ^ W) & s.W;
					long kingMask = (s.W^W) & W;
					boolean isDouble = ((kingMask & K)>0 || (moveMask & 0x780000000L)==0);
					if((s.DoPossibleJumpPieces()& moveMask) > 0 && isDouble){
						ArrayList tmp = s.createJumpStates();
						for(int j=0; j<tmp.size();j++){
							State temp = (State)tmp.get(j);
							if ((temp.W & moveMask) == 0){
								states.add(temp);
								deleteVal = true;
							}
						}
					}//end found possibles if
					if(deleteVal) {states.remove(i); i--;}
				}
			}
		for (int y = 0; y<states.size(); y++)
		{
			((State)states.get(y)).flipMove();
		}
		return states;
	}
	private void setup(){
		if(whiteMove){
			FA = W&K;
			BA = W |(W&K);
			FP = B | (B&K);
			BP = B & K;
		}
		else{
			FA = B | (B&K);
			BA = B & K;
			FP = W & K;
			BP = W |(W&K);
		}
		EMPTY = ~(B|W | (~MASK));
/*		if(DEBUG==1){
			System.out.println("\n******** State ********* \nFA: "+ Long.toBinaryString(FA) + 
					"\nBA: " + Long.toBinaryString(BA)+ 
					"\nFP: " + Long.toBinaryString(FP) + 
					"\nBP: "+ Long.toBinaryString(BP) +
					"\nEmpty: " + Long.toBinaryString(EMPTY));
		}*/
	}
	
	private void possibleSingleMoves(){
		LB = ((BA >> 5 & MAX) & EMPTY) & MASK;
		RB = ((BA >> 4 & MAX) & EMPTY) & MASK;
		RF = ((FA << 4 & MAX) & EMPTY) & MASK;
		LF = ((FA << 5 & MAX) & EMPTY) & MASK;
		if (DEBUG == 1) {
			
			System.out.println("\nNON_JUMPS ------------------------ \nBlack Pos: "+ Long.toBinaryString(B) + 
					" \nWhite Pos: " + Long.toBinaryString(W)+ 
					" \nK: " + Long.toBinaryString(K) + 
					" \nRF Jumps: "+ Long.toBinaryString(RF) +
					" \nLF Jumps: " + Long.toBinaryString(LF)+
					" \nRB Jumps: "+ Long.toBinaryString(RB)+
					" \nLB Jumps: " + Long.toBinaryString(LB));
		}
	}
	
	private int doJumps(){
		RF = ((( (FA << 4) & BP) << 4)& EMPTY) & MASK;
		RF = RF | ((( (FA << 4) & FP) << 4)& EMPTY) & MASK;
		LF = ((( (FA << 5) & BP) << 5)& EMPTY) & MASK;
		LF = LF | ((( (FA << 5) & FP) << 5)& EMPTY) & MASK;
		RB = ((( (BA >> 4) & FP) >> 4)& EMPTY) & MASK;
		LB = ((( (BA >> 5) & FP) >> 5)& EMPTY) & MASK;
		RB = RB | ((( (BA >> 4) & BP) >> 4)& EMPTY) & MASK;
		LB = LB | ((( (BA >> 5) & BP) >> 5)& EMPTY) & MASK;
		
		if (DEBUG == 1) {
			
			System.out.println("\nJUMPS------------------------ \nBlack Pos: "+ Long.toBinaryString(B) + 
					" \nWhite Pos: " + Long.toBinaryString(W)+ 
					" \nK: " + Long.toBinaryString(K) + 
					" \nRF Jumps: "+ Long.toBinaryString(RF) +
					" \nLF Jumps: " + Long.toBinaryString(LF)+
					" \nRB Jumps: "+ Long.toBinaryString(RB)+
					" \nLB Jumps: " + Long.toBinaryString(LB));
		}
		if ((RF | LF | RB | LB) > 0 )return 1; 
		else return 0; 
	}
	
	private long DoPossibleJumpPieces(){
		RF = ((( (FA << 4) & BP) << 4)& EMPTY) & MASK;
		RF = RF | ((( (FA << 4) & FP) << 4)& EMPTY) & MASK;
		LF = ((( (FA << 5) & BP) << 5)& EMPTY) & MASK;
		LF = LF | ((( (FA << 5) & FP) << 5)& EMPTY) & MASK;
		RB = ((( (BA >> 4) & FP) >> 4)& EMPTY) & MASK;
		LB = ((( (BA >> 5) & FP) >> 5)& EMPTY) & MASK;
		RB = RB | ((( (BA >> 4) & BP) >> 4)& EMPTY) & MASK;
		LB = LB | ((( (BA >> 5) & BP) >> 5)& EMPTY) & MASK;
		
		
/*		if (DEBUG == 1) {
			
			System.out.println("\nJUMPS------------------------ \nBlack Pos: "+ Long.toBinaryString(B) + 
					" \nWhite Pos: " + Long.toBinaryString(W)+ 
					" \nK: " + Long.toBinaryString(K) + 
					" \nRF Jumps: "+ Long.toBinaryString(RF) +
					" \nLF Jumps: " + Long.toBinaryString(LF)+
					" \nRB Jumps: "+ Long.toBinaryString(RB)+
					" \nLB Jumps: " + Long.toBinaryString(LB));
		}*/
		return ((RF>>8)|(LF>>10)|(RB<<8)|(LB<<10)); 
	}
	
	private ArrayList createSingleStates(){
			long nFA; // black on black's turn, white kings on white's turn
			long nBA; // white on white's turn, black kings on black's turn
			long nFP; // fowards passive (i.e. black on white's turn, white kings on black's turn 
			long nBP; // backwards passive (i.e. white on black's turn, black kings on white's move
			long nK = K;
			long nB = B;
			long nW = W;
		
			long leftMask = 0xFFFFC0000L;
			long rightMask =0x3FFFFL;
			ArrayList newStates = new ArrayList();
			int index;
		
			// LEFT FORWARD JUMPS
			if((LF & leftMask) >0){
				long sb = 0x800000000L; // the single bit to shift, starting at 36th bit
				for(int i=18;i>=0;i--){
					long mask = sb & LF;
					if(mask > 1){
						long negaMask = ~mask; // get negation orig mask
						negaMask = negaMask >> 5; // get negation mask for jumped pos 
						nFA = FA & negaMask | mask;
						nBA = BA & negaMask | mask;
						nK = (K & negaMask) | mask;
						if(whiteMove){
							if ((nFA & nK) > 0){  // white king makes a jump
								nW = nBA |nFA; // moves the white king 
								nB = B;
							//	nK = (K & negaMask) | mask;
							}
						}
						
						else{
							// Black jumps with normal piece
								if((K&nFA) == (K & FA))
									nK = K ;
							//	else
							//		nK = (K & negaMask) | mask;
								nW = BP;
								nB = nFA;
							long lastRow = 0x780000000L;
							if((nB & lastRow) > 0)
								nK = K | (nB & lastRow);
						}
						
						State newState = new State(nW,nB,nK,!whiteMove);
						newStates.add(newState);
					  }
					  sb = sb >>1;
					}
				} // end left side
			if((LF & rightMask) > 0){
				long sb = 0x10000L; // the single bit to shift, starting at 17th bit
				for(int i=17;i>=0;i--){
					long mask = sb & LF;
					if(mask > 1){
						long negaMask = ~mask; // get negation orig mask
						negaMask = negaMask >> 5; // get negation mask for jumped pos 
						nFA = FA & negaMask | mask;
						nBA = BA & negaMask | mask;
						nK = (K & negaMask) | mask;
						if(whiteMove){
							if ((nFA & nK) > 0){  // white king makes a jump
								nW = nBA |nFA; // moves the white king 
								nB = B;
							//	nK = (K & negaMask) | mask;
							}
						}
						else{
							// Black jumps with normal piece
								if((K&nFA) == (K & FA))
									nK = K ;
							//	else
							//		nK = (K & negaMask) | mask;
								nW = BP;
								nB = nFA ;
						}
						State newState = new State(nW,nB,nK,!whiteMove);
						newStates.add(newState);
					  }
					  sb = sb >>1;
					}
				} // end right side
				// END LEFT FORWARD JUMPS
			
			
				// RIGHT FORWARD JUMPS
			if((RF & leftMask) >0){
							long sb = 0x800000000L; // the single bit to shift, starting at 36th bit
							for(int i=18;i>=0;i--){
								long mask = sb & RF;
								if(mask > 1){
									long negaMask = ~mask; // get negation orig mask
									negaMask = negaMask >> 4; // get negation mask for jumped pos 
									nFA = FA & negaMask | mask;
									nBA = BA & negaMask | mask;
									nK = (K & negaMask) | mask;
									if(whiteMove){
										if ((nFA & nK) > 0){  // white king makes a jump
											nW = nBA |nFA; // moves the white king 
											nB = B;
										//	nK = (K & negaMask) | mask;
										}
									}
									else{
										// Black jumps with normal piece
											if((K&nFA) == (K & FA))
												nK = K ;
											nW = BP;
											nB = nFA ;
										long lastRow = 0x780000000L;
										if((nB & lastRow) > 0)
											nK = K | (nB & lastRow);
									}
									State newState = new State(nW,nB,nK,!whiteMove);
									newStates.add(newState);
								  }
								  sb = sb >>1;
								}
							} // end left side
						if((RF & rightMask) > 0){
							long sb = 0x10000L; // the single bit to shift, starting at 17th bit
							for(int i=17;i>=0;i--){
								long mask = sb & RF;
								if(mask > 1){
									long negaMask = ~mask; // get negation orig mask
									negaMask = negaMask >> 4; // get negation mask for jumped pos 
									nFA = (FA & negaMask) | mask;
									nBA = BA & negaMask | mask;
									nK = (K & negaMask) | mask;
									if(whiteMove){
										if ((nFA & nK) > 0){  // white king makes a jump
											nW = nBA|nFA ; // moves the white king 
											nB = B;
											
										}
									}
									else{
										// Black jumps with normal piece
											if((K&nFA) == (K & FA))
												nK = K ;
									//		else
									//			nK = (K & negaMask) | mask;
											nW = BP;
											nB = nFA;

									}
									State newState = new State(nW,nB,nK,!whiteMove);
									newStates.add(newState);
								  }
								  sb = sb >>1;
								}
						} // end right side	
			
			// RIGHT BACKWARD JUMPS
			if((RB & leftMask) > 0){
				long sb = 0x800000000L; // the single bit to shift, starting at 36th bit
				for(int i=18;i>=0;i--){
					long mask = sb & RB;
					if(mask > 1){
						long negaMask = ~mask; // get negation orig mask
						negaMask = (negaMask << 4)  | 0xFL; // get negation mask for jumped pos 
						nBA = BA & negaMask | mask;
						nK = (K & negaMask) | mask;	
						nFA = FA & negaMask |mask;
						if(whiteMove){
							if((BA & K) == (nBA &K))
								nK = K;								
							nW = nBA; // moves the white piece forward 
							nB = B; // removes black piece
							}
						else{
							if((nBA & nK) > 0){ // Black jumps with a king
								nW = W;
								nB = nBA |nFA;
							}
						}	
					  State newState = new State(nW,nB,nK,!whiteMove);
					  newStates.add(newState);
				  
					}
				sb = sb >>1;
				} // end left side
			}
			if((RB & rightMask) >0){
						long sb = 0x10000L; // the single bit to shift, starting at 36th bit
						for(int i=18;i>=0;i--){
							long mask = sb & RB;
								if(mask > 1){
								long negaMask = ~mask; // get negation orig mask
								negaMask = (negaMask << 4)  | 0xFL; // get negation mask for jumped pos 
								nBA = BA & negaMask | mask;
								nK = (K & negaMask) | mask;	
								nFA = FA & negaMask | mask;
								if(whiteMove){
									if((BA & K) == (nBA &K))
										nK = K;
									nW = nBA; // moves the white piece forward 
									nB = B; // removes black piece
									long firstRow = 0xFL;
									if((nW & firstRow) > 0)
									 	nK = K | (nW & firstRow);
								}
								else{
									if((nBA & nK) > 0){ // Black jumps with a king
										nW = W;
										nB = nBA|nFA;
									}
								}	
							  State newState = new State(nW,nB,nK,!whiteMove);
							  newStates.add(newState);
										  
							}
						sb = sb >>1;
						}
				}// end right side
			
			
				// BACKWARDS LEFT JUMPS
				 if((LB & leftMask) >0){
					 long sb = 0x800000000L; // the single bit to shift, starting at 36th bit
					 for(int i=18;i>=0;i--){
						 long mask = sb & LB;
						 if(mask > 1){
							 long negaMask = ~mask; // get negation orig mask
							 negaMask = (negaMask << 5)  | 0x1FL; // get negation mask for jumped pos 
							 nBA = BA & negaMask | mask;
							 nK = (K & negaMask) | mask;
							 nFA = FA & negaMask | mask;
							 if(whiteMove){
								if((BA & K) == (nBA &K))
									nK = K;
								nW = nBA; // moves the white piece forward 
								nB = B; // removes black piece
								
							 }
							 else{
								 if((nBA & nK) > 0){ // Black jumps with a king
									  // updates black king pos, removes a jumped white king
									 nW = W;
									 nB = nBA |nFA ;
								 }
						   }								 
						   State newState = new State(nW,nB,nK,!whiteMove);
						   //BitBoard new1 = BitBoard(nW,nB,nK,whiteMove,Hashy);
						   newStates.add(newState);
					   
						 }
						 sb = sb >>1;
					 } 
				 }// end left side
					 if((LB & rightMask) >0){
								 long sb = 0x10000L; // the single bit to shift, starting at 36th bit
								 for(int i=18;i>=0;i--){
									 long mask = sb & LB;
									if(mask > 1){
										long negaMask = ~mask; // get negation orig mask
										negaMask = (negaMask << 5)  | 0x1FL; // get negation mask for jumped pos 
										nBA = BA & negaMask | mask;
										nK = (K & negaMask) | mask;
										nFA = FA & negaMask | mask;
										if(whiteMove){
											if((BA & K) == (nBA &K))
												nK = K;
											nW = nBA ; // moves the white piece forward 
											nB = B; // removes black piece
											long firstRow = 0xFL;
											if((nW & firstRow) > 0)
											  nK = K | (nW & firstRow);	
										}
										else{
											if((nBA & nK) > 0){ // Black jumps with a king
												nW = W;
												nB = nBA|nFA;
											}
									  }									 
									   State newState = new State(nW,nB,nK,!whiteMove);
									   newStates.add(newState);
								   
									 }
									sb = sb >>1;
								 } 	
					 }// end right side
			
				if(DEBUG==1){
					System.out.println("\nNUMBER OF CREATED STATES: " + newStates.size());
				}
			return newStates;
		}
		
		
		
		
		
	private ArrayList createJumpStates(){
			long nFA; // black on black's turn, white kings on white's turn
			long nBA; // white on white's turn, black kings on black's turn
			long nFP; // fowards passive (i.e. black on white's turn, white kings on black's turn 
			long nBP; // backwards passive (i.e. white on black's turn, black kings on white's move
			long nK = K;
			long nB = B;
			long nW = W;
		
			long leftMask = 0xFFFFC0000L;
			long rightMask =0x3FFFFL;
			ArrayList newStates = new ArrayList();
			int index;
		
			// LEFT FORWARD NON JUMPS
			if((LF & leftMask) >0){
				long sb = 0x800000000L; // the single bit to shift, starting at 36th bit
				for(int i=18;i>=0;i--){
					long mask = sb & LF;
					if(mask > 1){
						long negaMask = ~mask; // get negation orig mask
						long firstShift = negaMask >> 5; // get negation mask for jumped pos 
						nBP = BP & firstShift; 
						nFP = FP & firstShift;
						negaMask = firstShift >> 5; // get negation mask for original pos
						nBA = BA & negaMask |mask;
						nFA = FA & negaMask |mask;
						nK = (K & firstShift & negaMask) | mask;
						if(whiteMove){
							if ((nFA & nK) > 0){  // white king makes a jump
								nW = nFA |nBA; // moves the white king 
								nB = nFP; // removes black piece
							}
						}
						else{
							// Black jumps with normal piece
								if((K&nFA) == (K & FA))
									nK = K & firstShift;
								nW = nBP;
								nB = nFA;
								long lastRow = 0x780000000L;
								if((nB & lastRow) > 0)
									nK = K | (nB & lastRow);
						}
						State newState = new State(nW,nB,nK,whiteMove);
						newStates.add(newState);
					  }
					  sb = sb >>1;
					}
				} // end left side
			if((LF & rightMask) > 0){
				long sb = 0x10000L; // the single bit to shift, starting at 17th bit
				for(int i=17;i>=0;i--){
					long mask = sb & LF;
					if(mask > 1){
						long negaMask = ~mask; // get negation orig mask
						long firstShift = negaMask >> 5; // get negation mask for jumped pos 
						nBP = BP & firstShift; 
						nFP = FP & firstShift; 
						negaMask = firstShift >> 5; // get negation mask for original pos
						nFA = FA & negaMask |mask;
						nBA = BA & negaMask |mask;
						nK = (K & negaMask & firstShift) | mask;
						if(whiteMove){
							if ((nFA & nK) > 0){  // white king makes a jump
								nW = nFA | nBA ; // moves the white king 
								nB = nFP; // removes black piece
							}
						}
						else{
							// Black jumps with normal piece
								if((K&nFA) == (K & FA))
									nK = K & firstShift;
								else
									nK = (K & firstShift & negaMask) | mask;
								nW = nBP;
								nB = nFA ;
						}
						State newState = new State(nW,nB,nK,whiteMove);
						newStates.add(newState);
					  }
					  sb = sb >>1;
					}
				} // end right side
				// END LEFT FORWARD JUMPS
			
			
				// RIGHT FORWARD JUMPS
			if((RF & leftMask) >0){
						long sb = 0x800000000L; // the single bit to shift, starting at 36th bit
						for(int i=18;i>=0;i--){
							long mask = sb & RF;
							if(mask > 1){
								long negaMask = ~mask; // get negation orig mask
								long firstShift = negaMask >> 4; // get negation mask for jumped pos 
								nBP = BP & firstShift; 
								nFP = FP & firstShift; 
								negaMask = firstShift >> 4; // get negation mask for original pos
								nFA = FA & negaMask | mask;
								nBA = BA & negaMask | mask;
								nK = (K & negaMask & firstShift) | mask;
								if(whiteMove){
									if ((nFA & nK) > 0){  // white king makes a jump
										nW = nBA |nFA; // moves the white king 
										nB = nFP; // removes black piece
										
									}
								}
								else{
									// Black jumps with normal piece
										if((K&nFA) == (K & FA))
											nK = K & firstShift;
										nW = nBP;
										nB = nFA ;
									long lastRow = 0x780000000L;
									if((nB & lastRow) > 0)
										nK = K | (nB & lastRow);
								}
								State newState = new State(nW,nB,nK,whiteMove);
								newStates.add(newState);
							  }
							  sb = sb >>1;
							}
						} // end left side
					
					if((RF & rightMask) > 0){
						long sb = 0x10000L; // the single bit to shift, starting at 17th bit
						for(int i=17;i>=0;i--){
							long mask = sb & RF;
							if(mask > 1){
								long negaMask = ~mask; // get negation orig mask
								long firstShift = negaMask >> 4; // get negation mask for jumped pos 
								nBP = BP & firstShift; 
								nFP = FP & firstShift; 
								negaMask = firstShift >> 4; // get negation mask for original pos
								nFA = FA & negaMask | mask;
								nBA = BA & negaMask | mask;
								nK = (K & negaMask & firstShift) | mask;
								if(whiteMove){
									if ((nFA & nK) > 0){  // white king makes a jump
										nW = nBA |nFA; // moves the white king 
										nB = nFP; // removes black piece
									}
								}
								else{
									// Black jumps with normal piece
										if((K&nFA) == (K & FA))
											nK = K & firstShift;
										nW = nBP;
										nB = nFA ;
								}
								State newState = new State(nW,nB,nK,whiteMove);
								newStates.add(newState);
							  }
							  sb = sb >>1;
							}
						} // end right side	
			
			// RIGHT BACKWARD JUMPS
			if((RB & leftMask) > 0){
				long sb = 0x800000000L; // the single bit to shift, starting at 36th bit
				for(int i=18;i>=0;i--){
					long mask = sb & RB;
					if(mask > 1){
						long negaMask = ~mask; // get negation orig mask
						long firstShift = (negaMask << 4 ) | 0xFL; // get negation mask for jumped pos 
						nBP = BP & firstShift; 
						nFP = FP & firstShift; 
						negaMask = (firstShift << 4) | 0xFL; // get negation mask for original pos
						nBA = BA & negaMask | mask;
						nFA = (FA & negaMask) | mask;
						nK = (K & negaMask & firstShift) | mask;
						if(whiteMove){
							if( (nBA & K) == (BA & K))
								nK = K & firstShift;
							nW = nBA | mask; // moves the white piece forward 
							nB = nFP; // removes black piece
							
						}
						else{
							if((nBA & nK) > 0){ // Black jumps with a king
								nW = nBP;
								nB = nFA;
							}
						}	
					  State newState = new State(nW,nB,nK,whiteMove);
					  newStates.add(newState);
				  
					}
				sb = sb >>1;
				} // end left side
			}
			if((RB & rightMask) >0){
						long sb = 0x10000L; // the single bit to shift, starting at 36th bit
						for(int i=18;i>=0;i--){
							long mask = sb & RB;
							if(mask > 1){
								long negaMask = ~mask; // get negation orig mask
								long firstShift = (negaMask << 4 ) | 0xFL; // get negation mask for jumped pos 
								nBP = BP & firstShift; 
								nFP = FP & firstShift; 
								negaMask = (firstShift << 4)| 0xFL; // get negation mask for original pos
								nBA = BA & negaMask | mask;
								nFA = (FA & negaMask) |mask;
								nK = (K & negaMask & firstShift) | mask;
								if(whiteMove){
									if( (nBA & K) == (BA & K))
										nK = K & firstShift;
									nW = nBA; // moves the white piece forward 
									nB = nFP; // removes black piece
									long firstRow = 0xFL;
									if((nW & firstRow) > 0)
									  nK = K | (nW & firstRow);
								}
								else{
									if((nBA & nK) > 0){ // Black jumps with a king
										nW = nBP;
										nB = nFA;

									}
							  } 							
							  State newState = new State(nW,nB,nK,whiteMove);
							  newStates.add(newState);
						  
							}
							sb = sb >>1; 
						}
				}// end right side
			
			
				// BACKWARDS LEFT JUMPS
				 if((LB & leftMask) >0){
					 long sb = 0x800000000L; // the single bit to shift, starting at 36th bit
					 for(int i=18;i>=0;i--){
						 long mask = sb & LB;
						 if(mask > 1){
							 long negaMask = ~mask; // get negation orig mask
							 long firstShift = (negaMask << 5 ) | 0x1FL; // get negation mask for jumped pos 
							 nBP = BP & firstShift; 
							 nFP = FP & firstShift; 
							 negaMask = (firstShift << 5) | 0x1FL; // get negation mask for original pos
							 nBA = (BA & negaMask) | mask;
							 nFA = (FA & negaMask) | mask;
							 nK = (K & negaMask & firstShift) | mask;
							 if(whiteMove){
								if( (nBA & K) == (BA & K))
									nK = K & firstShift;
								nW = nBA; // moves the white piece forward 
								nB = nFP; // removes black piece
							 }
							 else{
								 if((nBA & nK) > 0){ // Black jumps with a king
									// nK = (K & negaMask & firstShift) | mask; // updates black king pos, removes a jumped white king
									 nW = nBP;
									 nB = nFA;
								 }
						   }								 
						   State newState = new State(nW,nB,nK,whiteMove);
						   if(DEBUG==1) System.out.println(newState.toString());
						   //BitBoard new1 = BitBoard(nW,nB,nK,whiteMove,Hashy);
						   newStates.add(newState);
					   
						 }
						 sb = sb >>1;
					 } 
				 }// end left side
					 if((LB & rightMask) >0){
								 long sb = 0x10000L; // the single bit to shift, starting at 36th bit
								 for(int i=18;i>=0;i--){
									 long mask = sb & LB;
									 if(mask > 1){
										 long negaMask = ~mask; // get negation orig mask
										 long firstShift = (negaMask << 5 ) | 0x1FL; // get negation mask for jumped pos 
										 nBP = BP & firstShift; 
										 nFP = FP & firstShift; 
										 negaMask = (firstShift << 5) | 0x1FL; // get negation mask for original pos
										 nBA = BA & negaMask | mask;
										 nFA = FA & negaMask | mask;
										 nK =(K & negaMask & firstShift) | mask;
										 if(whiteMove){
											if( (nBA & K) == (BA & K))
												nK = K & firstShift;
											nW = nBA; // moves the white piece forward 
											nB = nFP; // removes black piece
											long firstRow = 0xFL;
											if((nW & firstRow) > 0)
											  nK = K | (nW & firstRow);
										 }
										 else{
											 if((nBA & nK) > 0){ // Black jumps with a king
												 nW = nBP;
												 nB = nFA;
											 }
									   }									 
									   State newState = new State(nW,nB,nK,whiteMove);
									   newStates.add(newState);
								   
									 }
									sb = sb >>1;
								 } 	
					 }// end right side
			
//				if(DEBUG==1){
//					System.out.println("\nNUMBER OF CREATED STATES: " + newStates.size());
//				}
			return newStates;
		}
		
	public static void outputNextStates(BitBoard board){
			for(int i=0; i<board.nStates.size();i++){
				State nState = (State)board.nStates.get(i);
				System.out.print(nState.toString());
			}
		}
	

}
